@@ -12,17 +12,20 @@ from scipy.io.matlab.miobase import MatReadError
 from kilosort.gui.logger import setup_logger
 from kilosort.gui.minor_gui_elements import ProbeBuilder, create_prb
 from kilosort.io import load_probe, BinaryRWFile
-from kilosort.parameters import MAIN_PARAMETERS, EXTRA_PARAMETERS
+from kilosort.parameters import MAIN_PARAMETERS, EXTRA_PARAMETERS, compare_settings
 
 
 logger = setup_logger(__name__)
 
 _DEFAULT_DTYPE = 'int16'
 _ALLOWED_FILE_TYPES = ['.bin', '.dat', '.bat', '.raw']  # For binary data
+_PROBE_SETTINGS = [
+    'nearest_chans', 'dmin', 'dminx', 'max_channel_distance', 'x_centers'
+    ]
 
 class SettingsBox(QtWidgets.QGroupBox):
     settingsUpdated = QtCore.Signal()
-    previewProbe = QtCore.Signal(object)
+    previewProbe = QtCore.Signal()
     dataChanged = QtCore.Signal()
 
     def __init__(self, parent):
@@ -31,6 +34,7 @@ class SettingsBox(QtWidgets.QGroupBox):
         self.gui = parent
         self.load_enabled = False
         self.use_file_object = False
+        self.path_check = None
         
         self.select_data_file = QtWidgets.QPushButton("Select Binary File")
         self.data_file_path = self.gui.data_path
@@ -98,6 +102,7 @@ class SettingsBox(QtWidgets.QGroupBox):
 
         self.extra_parameters_window = ExtraParametersWindow(self)
         self.extra_parameters_button = QtWidgets.QPushButton('Extra settings')
+        self.revert_parameters_button = QtWidgets.QPushButton('Use default settings')
         self.import_settings_button = QtWidgets.QPushButton('Import')
         self.export_settings_button = QtWidgets.QPushButton('Export')
 
@@ -202,7 +207,7 @@ class SettingsBox(QtWidgets.QGroupBox):
 
         row_count += rspan
         layout.addWidget(self.probe_layout_selector, row_count, col1, rspan, dbl)
-        self.probe_layout_selector.currentTextChanged.connect(
+        self.probe_layout_selector.textActivated.connect(
             self.on_probe_layout_selected
         )
 
@@ -246,14 +251,22 @@ class SettingsBox(QtWidgets.QGroupBox):
                 )
             inp = getattr(self, f'{k}_input')
             inp.editingFinished.connect(self.update_parameter)
+            if k in _PROBE_SETTINGS:
+                inp.editingFinished.connect(self.show_probe_layout())
 
         row_count += rspan
         layout.addWidget(
-            self.extra_parameters_button, row_count, col1, rspan, dbl
+            self.extra_parameters_button, row_count, col1, rspan, cspan1
             )
         self.extra_parameters_button.clicked.connect(
             lambda x: self.extra_parameters_window.show()
             )
+        layout.addWidget(
+            self.revert_parameters_button, row_count, col2, rspan, cspan2
+        )
+        self.revert_parameters_button.clicked.connect(
+            self.set_default_field_values
+        )
         
         row_count += rspan
         layout.addWidget(
@@ -438,16 +451,17 @@ class SettingsBox(QtWidgets.QGroupBox):
             self.disable_load()
 
     def on_data_file_path_changed(self):
+        self.path_check = None
         data_file_path = Path(self.data_file_path_input.text())
         try:
             assert self.check_valid_binary_path(data_file_path)
+            self.data_file_path = data_file_path
+            self.gui.qt_settings.setValue('data_file_path', data_file_path)
 
             parent_folder = data_file_path.parent
             results_folder = parent_folder / "kilosort4"
             self.results_directory_input.setText(results_folder.as_posix())
             self.results_directory_input.editingFinished.emit()
-            self.data_file_path = data_file_path
-            self.gui.qt_settings.setValue('data_file_path', data_file_path)
 
             if self.check_settings():
                 self.enable_load()
@@ -458,20 +472,28 @@ class SettingsBox(QtWidgets.QGroupBox):
             self.disable_load()
 
     def check_valid_binary_path(self, filename):
+        if self.path_check is not None:
+            # Flag is set to False when path changes, this is to avoid checking
+            # the path repeatedly for no reason.
+            return self.path_check
+
         if filename is None:
             print('Binary path is None.')
-            return False
+            check = False
         else:
             f = Path(filename)
             if f.exists() and f.is_file():
                 if f.suffix in _ALLOWED_FILE_TYPES or self.use_file_object:
-                    return True
+                    check = True
                 else:
                     print(f'Binary file has invalid suffix. Must be {_ALLOWED_FILE_TYPES}')
-                    return False
+                    check = False
             else:
                 print('Binary file does not exist at that path.')
-                return False
+                check = False
+        
+        self.path_check = check
+        return check
 
     def disable_all_input(self, value):
         for button in self.buttons:
@@ -502,22 +524,34 @@ class SettingsBox(QtWidgets.QGroupBox):
             "data_dtype": self.data_dtype,
             }
         for k in list(MAIN_PARAMETERS.keys()):
-            self.settings[k] = getattr(self, k)
+            v = getattr(self, k)
+            self.settings[k] = v
+            self.update_setting_color(k, v, MAIN_PARAMETERS)
         for k in list(EXTRA_PARAMETERS.keys()):
-            self.settings[k] = getattr(self.extra_parameters_window, k)
+            v = getattr(self.extra_parameters_window, k)
+            self.settings[k] = v
+            self.update_setting_color(k, v, EXTRA_PARAMETERS, extras=True)
 
         if not self.check_valid_binary_path(self.data_file_path):
             return False
 
         none_allowed = [
-            'dmin', 'nt0min', 'max_channel_distance', 'x_centers',
-            'shift', 'scale'
+            'dmin', 'nt0min', 'x_centers', 'shift', 'scale', 'max_channel_distance'
             ]
         for k, v in self.settings.items():
             if v is None and k not in none_allowed:
-                print(f'`None` not allowed for parameter {k}.')
+                logger.info(f'`None` not allowed for parameter {k}.')
                 return False
         return True
+
+    def update_setting_color(self, k, v, d, extras=False):
+        o = self.extra_parameters_window if extras else self
+        if v != d[k]['default']:
+            c = "color : yellow"
+        else:
+            c = "color : white"
+        getattr(o, f'{k}_text').setStyleSheet(c)
+        getattr(o, f'{k}_input').setStyleSheet(c)
     
     @QtCore.Slot()
     def update_parameter(self):
@@ -541,15 +575,15 @@ class SettingsBox(QtWidgets.QGroupBox):
 
     def get_probe_template_args(self):
         epw = self.extra_parameters_window
-        template_args = [
-            epw.nearest_chans, epw.dmin, epw.dminx, 
-            epw.max_channel_distance, epw.x_centers, self.gui.device
-            ]
+        template_args = [getattr(epw, k) for k in _PROBE_SETTINGS]
         return template_args
 
     @QtCore.Slot()
     def show_probe_layout(self):
-        self.previewProbe.emit(self.get_probe_template_args())
+        if self.check_settings:
+            self.previewProbe.emit()
+        else:
+            logger.info("Cannot preview probe layout, invalid settings.")
 
     @QtCore.Slot(str)
     def on_probe_layout_selected(self, name):
@@ -699,7 +733,7 @@ class SettingsBox(QtWidgets.QGroupBox):
         # Trigger update so that probe layout in main gets updated, then
         # refresh probe view.
         self.update_settings()
-        self.previewProbe.emit(self.get_probe_template_args)
+        self.previewProbe.emit()
 
 
     def on_data_dtype_selected(self, data_dtype):
@@ -793,6 +827,7 @@ class SettingsBox(QtWidgets.QGroupBox):
     def reset(self):
         self.data_file_path_input.clear()
         self.data_file_path = None
+        self.path_check = None
         self.gui.qt_settings.setValue('data_file_path', None)
         self.results_directory_input.clear()
         self.results_directory_path = None
@@ -849,6 +884,8 @@ class ExtraParametersWindow(QtWidgets.QWidget):
             layout.addWidget(getattr(self, f'{k}_input'), row_count, col+3, 1, 2)
             inp = getattr(self, f'{k}_input')
             inp.editingFinished.connect(self.update_parameter)
+            if k in _PROBE_SETTINGS:
+                inp.editingFinished.connect(self.main_settings.show_probe_layout)
 
         self.setLayout(layout)
 
