@@ -190,7 +190,7 @@ def yweighted(yc, iC, adist, xy, device=torch.device('cuda')):
     return yct
 
 def run(ops, bfile, device=torch.device('cuda'), progress_bar=None,
-        clear_cache=False):        
+        clear_cache=False, verbose=False):        
     sig = ops['settings']['min_template_size']
     nsizes = ops['settings']['template_sizes'] 
 
@@ -243,9 +243,11 @@ def run(ops, bfile, device=torch.device('cuda'), progress_bar=None,
     logger.info('Detecting spikes...')
     prog = tqdm(np.arange(bfile.n_batches), miniters=200 if progress_bar else None, 
                 mininterval=60 if progress_bar else None)
+    # repeat performance log after every 10 minutes of data
+    log_skip = int(600 / (ops['batch_size'] / ops['fs']))
     try:
         for ibatch in prog:
-            if ibatch % 100 == 0:
+            if ibatch % log_skip == 0:
                 log_performance(logger, 'debug', f'Batch {ibatch}')
 
             X = bfile.padded_batch_to_torch(ibatch, ops)
